#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <syslog.h>
#include <string.h>
#include <time.h>
#include <sys/stat.h>

#include "config.h"
#include "utils.h"
#include "rfcutils.h"
#include "pbbs_out.h"
#include "log.h"

extern int errno;

struct newsgroup_map	*newsl		= NULL;

/*
 *	Crash, log the reason
 */
 
void panic(const char *fmt, ...)
{
	va_list args;
	
	va_start(args, fmt);
	say(L_ERR, fmt, args);
	va_end(args);

	exit(1);
}

/*
 *	Pick BBS distribution for the newsgroups
 */

struct newsgroup_map *pick_distrib(char *newsgroups)
{
	struct newsgroup_map *m;

	for (m = newsl; (m); m = m->next) {
		if (strstr(newsgroups, m->newsgroup))
			return m;
	}

	return NULL;
}

/*
 *	Process a message
 */

int process(char *path)
{
	struct art_hdr art = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
	struct pb_hdr pb = { "", 0, "", "", "", "", "", "", "", 0 };
	struct newsgroup_map *nm;
	char *src;
	char *p;
	FILE *f;
	
	if ((f = fopen(path, "r")) == NULL) {
		say(L_ERR, "Couldn't open article %s: %s", 
			path, strerror(errno));
		return -1;
	}
	
	parse_hdrs(f, &art);
	if ((nm = pick_distrib(art.newsgroups))) {
		if ((src = map_address(art.fromaddr, EMAIL_SRC))) {
			pb.mtype = MT_B;
			hstrncpy(pb.from, strupr(strtok(src, "@")), CALLEN);
			hstrncpy(pb.frombbs, strupr(strtok(NULL, "@")), HRLEN);
			hstrncpy(pb.fromname, art.from, NAMELEN);
			strcpy(pb.toname, "News: ");
			hstrncat(pb.toname, art.newsgroups, NAMELEN);
			hstrncpy(pb.bid, (art.bid) ? art.bid : pbbsmid(art.mid, art.newsgroups), BIDLEN);
			if (art.x_bbs_to) {
				if ((p = strchr(art.x_bbs_to, '@'))) {
					*p = '\0';
					p++;
				} else
					p = nm->bbs;
				hstrncpy(pb.to, art.x_bbs_to, CALLEN);
				hstrncpy(pb.tobbs, p, HRLEN);
			} else {
				hstrncpy(pb.to, nm->to, CALLEN);
				hstrncpy(pb.tobbs, nm->bbs, HRLEN);
			}
			hstrncpy(pb.subj, art.subj, LINELEN);
			pb.time = 0; /* krhm... */
			say(L_ROUTING, "%s %s > %s => $%s %s@%s > %s@%s", 
				art.mid, art.from, art.newsgroups, pb.bid, pb.from, 
				pb.frombbs, pb.to, pb.tobbs);
			export(f, &pb, &art);
		} else
			say(L_R_ERR, "%s %s > %s: No source address mapping", 
				art.mid, art.from, art.newsgroups);
	} else
		say(L_R_ERR, "%s %s > %s: Group not forwarded", 
			art.mid, art.from, art.newsgroups);
	
	freeart(&art);
	
	if (fclose(f))
		say(L_ERR, "%s %s > %s: Couldn't close article %s: %s", 
			art.mid, art.from, art.newsgroups, path, strerror(errno));

	return 0;
}

/*
 *	Main
 */

int main(void)
{
	char s[PATHLEN], *p;
	
	openlog("nntp_rcv", LOG_NDELAY, LOG_LOCAL6);
	
	read_config();
	read_newsgroup_map(NEWSGROUP_MAP, 0, &newsl);
	
	say(L_START, "Started as %s: %s", callsign, hostname);
	
	umask(077); /* We don't want our temp files to be world readable */
	
	while (fgets(s, PATHLEN, stdin)) {
		while ((p = strchr(s, '\x0A')))
			*p = '\0';
		process(s);
	}
	
	say(L_START, "Stopped");
	closelog();
	
	return 0;
}

