#include <string.h>
#include <stdlib.h>

#include "config.h"
#include "rfcutils.h"

/*
 * Read one header line including possible continuation from fp.
 * (by Tomi Manninen, OH2BNS)
 */

static char *read_header(FILE *fp)
{
	char *line, buf[LINELEN+1];
	int c;

	line = NULL;
	while (1) {
		if (fgets(buf, LINELEN, fp) == NULL)
			break;
		if (line == NULL)
			line = calloc(sizeof(char), strlen(buf) + 1);
		else
			line = realloc(line, strlen(line) + strlen(buf) + 1);
		if (line == NULL)
			break;
		strcat(line, buf);
		if (line[0] == '\n') /* end of headers */
			break;
		c = fgetc(fp);
		if (c == EOF)
			break;
		ungetc(c, fp);
		if (c != ' ' && c != '\t')
			break;
	}
	return line;
}

/*
 *	Separate the actual address "user@host" from an rfc822 address
 *	(or some non-standard one). Formats handled:
 *
 *	user@host
 *	user@host (name)
 *	name <user@host>
 */

char *separate_addr(char *addr)
{
	char ad[ADDRLEN];
	static char s[ADDRLEN];
	char *p1, *p2;
	
	strncpy(s, addr, ADDRLEN);

	if ((p1 = strchr(s, '(')) < (p2 = strchr(s, ')')) && (p1)) {
		/* user@host (name) ==> Rip the (name) off */
		p2++;
		for (; (*p2); p2++)
			*p1++ = *p2;
		*p1++ = '\0';
	}
	
	if ((p1 = strchr(s, '<')) < (p2 = strchr(s, '>')) && (p1)) {
		/* name <user@host> ==> rip everything but user@host off */
		strncpy(ad, p1 + 1, p2 - p1 - 1);
	} else
		strcpy(ad, s);
	
	/* Remove whitespace */
	for (p1 = ad, p2 = s; (*p1); p1++)
		if (*p1 != ' ') {
			*p2 = *p1;
			p2++;
		}
	*p2++ = '\0';
	
	return s;
}

/*
 *	Get the value of the header, stripping whitespace from start, malloced
 */

char *hdrval(char *s)
{
	char *t;
	
	while (((*s == ' ') || (*s == '\t')) && (*s != '\0'))
		s++;
	t = strdup(s);
	
	return t;
}

/*
 *	Parse RFC881 message headers
 */

int parse_hdrs(FILE *f, struct art_hdr *art)
{
	char *s, *p;
	
	while ((s = read_header(f)) && (*s != '\n')) {

		while ((p = strchr(s, '\n')))
			*p = '\0';
		if (art->path == NULL && !strncasecmp(s, "Path:", 5)) {
			art->path = hdrval(&s[5]);
		} else if (art->from == NULL && !strncasecmp(s, "From:", 5)) {
			art->from = hdrval(&s[5]);
		} else if (art->to == NULL && !strncasecmp(s, "To:", 3)) {
			art->to = hdrval(&s[3]);
		} else if (art->x_bbs_to == NULL && !strncasecmp(s, "X-BBS-To:", 9)) {
			art->x_bbs_to = hdrval(&s[3]);
		} else if (art->returnpath == NULL && !strncasecmp(s, "Return-Path:", 12)) {
			art->returnpath = hdrval(&s[12]);
		} else
		if (art->date == NULL && !strncasecmp(s, "Date:", 5)) {
			art->date = hdrval(&s[5]);
		} else
		if (art->newsgroups == NULL && !strncasecmp(s, "Newsgroups:", 11)) {
			art->newsgroups = hdrval(&s[11]);
		} else
		if (art->subj == NULL && !strncasecmp(s, "Subject:", 8)) {
			art->subj = hdrval(&s[8]);
		} else
		if (art->mid == NULL && !strncasecmp(s, "Message-ID:", 11)) {
			art->mid = hdrval(&s[11]);
		} else
		if (art->bid == NULL && !strncasecmp(s, "X-BID:", 6)) {
			art->bid = hdrval(&s[6]);
		} else
		if (art->bid == NULL && !strncasecmp(s, "X-MID:", 6)) {
			art->bid = hdrval(&s[6]);
		}
		free(s);
	}
	if (art->from)
		art->fromaddr = separate_addr(art->from);
	
	return (art->fromaddr && art->newsgroups && art->mid);
}

void freeart(struct art_hdr *art)
{
	free(art->path);
	free(art->from);
	free(art->to);
	free(art->returnpath);
	free(art->date);
	free(art->newsgroups);
	free(art->subj);
	free(art->mid);
	free(art->bid);
}
