#include <stdio.h>
#include <stdarg.h>
#include <syslog.h>
#include <string.h>
#include <sys/stat.h>

#include "config.h"
#include "utils.h"
#include "rfcutils.h"
#include "pbbs_out.h"
#include "log.h"

extern int errno;

/*
 *	Crash, log the reason
 */
 
void panic(const char *fmt, ...)
{
	va_list args;
	
	va_start(args, fmt);
	say(L_ERR, fmt, args);
	va_end(args);

	exit(1);
}

/*
 *	Process a message
 */

int process(char *fromaddr, char *tohost, char *toaddr)
{
	struct art_hdr art = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
	struct pb_hdr pb = { "", 0, "", "", "", "", "", "", "", 0 };
	char *src, *dst, *t;
	
	parse_hdrs(stdin, &art);
	
	if (!art.mid) {
		say(L_ERR, "No Message-Id: %s > %s", fromaddr, toaddr);
		freeart(&art);
		return -1;
	}
	
	if (!(t = map_address(fromaddr, EMAIL_SRC))) {
		say(L_R_ERR, "%s %s > %s: No source mapping", 
			art.mid, fromaddr, toaddr);
		freeart(&art);
		return -2;
	}
	src = strdup(t);
	
	if (!(t = map_address(toaddr, EMAIL_DST))) {
		say(L_R_ERR, "%s %s > %s: No dest mapping", 
			art.mid, fromaddr, toaddr);
		freeart(&art);
		return -2;
	}
	dst = strdup(t);
	
	pb.mtype = MT_P;
	hstrncpy(pb.from, strupr(strtok(src, "@")), CALLEN);
	hstrncpy(pb.frombbs, strupr(strtok(NULL, "@")), HRLEN);
	if (art.from)
		hstrncpy(pb.fromname, art.from, NAMELEN);
	if (art.to)
		hstrncat(pb.toname, art.to, NAMELEN);
	hstrncpy(pb.bid, (art.bid) ? art.bid : pbbsmid(art.mid, toaddr), BIDLEN);
	hstrncpy(pb.to, strupr(strtok(dst, "@")), CALLEN);
	hstrncpy(pb.tobbs, strupr(strtok(NULL, "@")), HRLEN);
	if (art.subj)
		hstrncpy(pb.subj, art.subj, LINELEN);
	pb.time = 0; /* krhm... */
	say(L_ROUTING, "%s %s > %s => $%s %s@%s > %s@%s", 
		art.mid, fromaddr, toaddr, pb.bid, pb.from, pb.frombbs, pb.to, pb.tobbs);
	export(stdin, &pb, &art);
	
	freeart(&art);
	
	return 0;
}

/*
 *	Main
 */

int main(int argc, char **argv)
{
	openlog("smtp_rcv", LOG_NDELAY, LOG_LOCAL6);
	read_config();
	umask(077); /* We don't want our temp files to be world readable */
	
	if (argc < 4) {
		say(L_ERR, "Wrong number (%i) of parameters. Usage: smtp_rcv fromaddr tohost toaddr", argc);
		return 1;
	}
	
	process(argv[1], argv[2], argv[3]);
	
	closelog();
	
	return 0;
}

